﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment.Algorithms {
  [Item("Single-objective Single Solution Context", "A context for single-objective single solution algorithms.")]
  public abstract class SingleObjectiveSingleSolutionContext<TSolutionScope> : SingleSolutionContext<TSolutionScope>
    where TSolutionScope: class, IScope {

    [Storable]
    private IValueParameter<DoubleValue> bestQuality;
    public double BestQuality {
      get { return bestQuality.Value.Value; }
      set { bestQuality.Value.Value = value; }
    }

    [StorableConstructor]
    protected SingleObjectiveSingleSolutionContext(bool deserializing) : base(deserializing) { }
    protected SingleObjectiveSingleSolutionContext(SingleObjectiveSingleSolutionContext<TSolutionScope> original, Cloner cloner)
      : base(original, cloner) {
      bestQuality = cloner.Clone(original.bestQuality);
    }
    protected SingleObjectiveSingleSolutionContext()
      : base() {
      Parameters.Add(bestQuality = new ValueParameter<DoubleValue>("BestQuality", new DoubleValue(double.NaN)));
    }
    protected SingleObjectiveSingleSolutionContext(string name) : base(name) {
      Parameters.Add(bestQuality = new ValueParameter<DoubleValue>("BestQuality", new DoubleValue(double.NaN)));
    }
    protected SingleObjectiveSingleSolutionContext(string name, ParameterCollection parameters)
      : base(name, parameters) {
      Parameters.Add(bestQuality = new ValueParameter<DoubleValue>("BestQuality", new DoubleValue(double.NaN)));
    }
    protected SingleObjectiveSingleSolutionContext(string name, string description)
      : base(name, description) {
      Parameters.Add(bestQuality = new ValueParameter<DoubleValue>("BestQuality", new DoubleValue(double.NaN)));
    }
    protected SingleObjectiveSingleSolutionContext(string name, string description, ParameterCollection parameters)
      : base(name, description, parameters) {
      Parameters.Add(bestQuality = new ValueParameter<DoubleValue>("BestQuality", new DoubleValue(double.NaN)));
    }
  }
}
